#!/usr/bin/env node
"use strict";
/// <reference types="./typing" />
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const bull_1 = __importDefault(require("bull"));
const chalk_1 = __importDefault(require("chalk"));
const vorpal_1 = __importDefault(require("vorpal"));
const searchjs_1 = require("searchjs");
const terminal_link_1 = __importDefault(require("terminal-link"));
const ms_1 = __importDefault(require("ms"));
exports.vorpal = new vorpal_1.default();
let queue;
const showJobs = (arr, filter) => {
    const data = arr.map(job => ({
        id: job.id,
        data: job.data,
        time: Number.isNaN(job.timestamp) ? job.timestamp : new Date(job.timestamp),
        name: job.name,
        failedReason: job.failedReason,
        stackTrace: job.stacktrace,
        returnValue: job.returnvalue,
        attemptsMade: job.attemptsMade,
        delay: job.delay,
        progress: job._progress
    }));
    const filteredData = searchjs_1.matchArray(data, filter);
    logArray(filteredData);
};
const checkQueue = async () => {
    if (!queue) {
        let err = new Error();
        err.stack = chalk_1.default.yellow("Need connect before");
        throw err;
    }
    return await queue.isReady();
};
const getJob = async (jobId) => {
    const job = await queue.getJob(jobId);
    if (!job) {
        let err = new Error();
        err.stack = chalk_1.default.yellow(`Job "${jobId}" not found`);
        throw err;
    }
    return job;
};
const getFilter = (filter) => {
    return new Promise(resolve => {
        try {
            resolve(JSON.parse(filter || "{}"));
        }
        catch (e) {
            let err = new Error();
            err.stack = chalk_1.default.yellow(`Error occured, seems passed "filter" incorrect json`);
            throw err;
        }
    });
};
const getTimeAgoFilter = (timeAgo) => {
    return new Promise(resolve => {
        try {
            const msAgo = timeAgo && timeAgo.length ? ms_1.default(timeAgo) : void 0;
            const filter = msAgo ? { time: { gte: Date.now() - msAgo } } : {};
            resolve(filter);
        }
        catch (e) {
            let err = new Error();
            err.stack = chalk_1.default.yellow(`Error occured, seems passed "timeAgo" incorrect`);
            throw err;
        }
    });
};
const logArray = (arr) => {
    console.dir(arr, {
        colors: true,
        depth: null,
        maxArrayLength: Infinity
    });
};
const searchjsLink = terminal_link_1.default("searchjs", "https://github.com/deitch/searchjs#examples");
const msLink = terminal_link_1.default("ms", "https://github.com/zeit/ms#examples");
exports.vorpal
    .command("connect <queue>", "connect to bull queue")
    .option("-p, --prefix <prefix>", "prefix to use for all queue jobs")
    .option("-r, --redis <redis>", "host:port of redis, default localhost:6379")
    .action(async ({ queue: name, options }) => {
    queue && queue.close();
    const url = options.redis
        ? `redis://${options.redis}`
        : "redis://localhost:6379";
    queue = bull_1.default(name, url, options);
    await queue.isReady();
    const prefix = options.prefix || "bull";
    console.log(chalk_1.default.green(`Connected to ${url}, prefix: ${prefix}, queue: ${name}`));
    exports.vorpal.delimiter(`BULL-REPL | ${prefix}.${name}> `).show();
});
exports.vorpal.command("stats", "count of jobs by groups").action(async () => {
    await checkQueue();
    console.table(await queue.getJobCounts());
});
exports.vorpal
    .command("active", "fetch active jobs")
    .option("-f, --filter <filter>", `filter jobs via ${searchjsLink}`)
    .option("-ta, --timeAgo <timeAgo>", `get jobs since time ago via ${msLink}`)
    .action(async ({ options }) => {
    await checkQueue();
    const filter = await getFilter(options.filter);
    const timeAgoFilter = await getTimeAgoFilter(options.timeAgo);
    showJobs(await queue.getActive(), { ...filter, ...timeAgoFilter });
});
exports.vorpal
    .command("waiting", "fetch waiting jobs")
    .option("-f, --filter <filter>", `filter jobs via ${searchjsLink}`)
    .option("-ta, --timeAgo <timeAgo>", `get jobs since time ago via ${msLink}`)
    .action(async ({ options }) => {
    await checkQueue();
    const filter = await getFilter(options.filter);
    const timeAgoFilter = await getTimeAgoFilter(options.timeAgo);
    showJobs(await queue.getWaiting(), { ...filter, ...timeAgoFilter });
});
exports.vorpal
    .command("completed", "fetch completed jobs")
    .option("-f, --filter <filter>", `filter jobs via ${searchjsLink}`)
    .option("-ta, --timeAgo <timeAgo>", `get jobs since time ago via ${msLink}`)
    .action(async ({ options }) => {
    await checkQueue();
    const filter = await getFilter(options.filter);
    const timeAgoFilter = await getTimeAgoFilter(options.timeAgo);
    showJobs(await queue.getCompleted(), { ...filter, ...timeAgoFilter });
});
exports.vorpal
    .command("failed", "fetch failed jobs")
    .option("-f, --filter <filter>", `filter jobs via ${searchjsLink}`)
    .option("-ta, --timeAgo <timeAgo>", `get jobs since time ago via ${msLink}`)
    .action(async ({ options }) => {
    await checkQueue();
    const filter = await getFilter(options.filter);
    const timeAgoFilter = await getTimeAgoFilter(options.timeAgo);
    showJobs(await queue.getFailed(), { ...filter, ...timeAgoFilter });
});
exports.vorpal
    .command("delayed", "fetch delayed jobs")
    .option("-f, --filter <filter>", `filter jobs via ${searchjsLink}`)
    .option("-ta, --timeAgo <timeAgo>", `get jobs since time ago via ${msLink}`)
    .action(async ({ options }) => {
    await checkQueue();
    const filter = await getFilter(options.filter);
    const timeAgoFilter = await getTimeAgoFilter(options.timeAgo);
    showJobs(await queue.getDelayed(), { ...filter, ...timeAgoFilter });
});
exports.vorpal.command("get <jobId>", "get job").action(async ({ jobId }) => {
    await checkQueue();
    const job = await getJob(jobId);
    showJobs([job], {});
});
exports.vorpal
    .command("add <data>", "add job to queue")
    .option("-n, --name <name>", "name for named job")
    .action(async function ({ data, options }) {
    await checkQueue();
    let jobData;
    try {
        jobData = JSON.parse(data);
    }
    catch (e) {
        let err = new Error();
        err.stack = chalk_1.default.yellow(`Error occured, seems "data" incorrect json`);
        throw err;
    }
    const answer = await this.prompt({
        name: "a",
        message: "Add? (y/n): "
    });
    if (answer.a !== "y") {
        return;
    }
    const jobName = options.name || "__default__";
    const addedJob = await queue.add(jobName, jobData);
    console.log(chalk_1.default.green(`Job with name '${jobName}', id '${addedJob.id}' added`));
});
exports.vorpal
    .command("rm <jobId>", "remove job")
    .action(async function ({ jobId }) {
    await checkQueue();
    const job = await getJob(jobId);
    const answer = await this.prompt({
        name: "a",
        message: "Remove? (y/n): "
    });
    if (answer.a !== "y") {
        return;
    }
    await job.remove();
    console.log(chalk_1.default.green(`Job "${jobId}" removed`));
});
exports.vorpal
    .command("retry <jobId>", "retry job")
    .action(async function ({ jobId }) {
    await checkQueue();
    const job = await getJob(jobId);
    const answer = await this.prompt({
        name: "a",
        message: "Retry? (y/n): "
    });
    if (answer.a !== "y") {
        return;
    }
    await job.retry();
    console.log(chalk_1.default.green(`Job "${jobId}" retried`));
});
exports.vorpal
    .command("promote <jobId>", "promote job")
    .action(async function ({ jobId }) {
    await checkQueue();
    const job = await getJob(jobId);
    const answer = await this.prompt({
        name: "a",
        message: "Promote? (y/n): "
    });
    if (answer.a !== "y") {
        return;
    }
    await job.promote();
    console.log(chalk_1.default.green(`Job "${jobId}" promoted`));
});
exports.vorpal
    .command("fail <jobId> <reason>", "fail job")
    .action(async function ({ jobId, reason }) {
    await checkQueue();
    const job = await getJob(jobId);
    const answer = await this.prompt({
        name: "a",
        message: "Fail? (y/n): "
    });
    if (answer.a !== "y") {
        return;
    }
    await job.moveToFailed({ message: reason }, true);
    console.log(chalk_1.default.green(`Job "${jobId}" failed`));
});
exports.vorpal
    .command("complete <jobId> <data>", "complete job")
    .action(async function ({ jobId, data }) {
    await checkQueue();
    const job = await getJob(jobId);
    let returnValue;
    try {
        returnValue = JSON.parse(data);
    }
    catch (e) {
        let err = new Error();
        err.stack = chalk_1.default.yellow(`Error occured, seems "data" incorrect json`);
        throw err;
    }
    const answer = await this.prompt({
        name: "a",
        message: "Complete? (y/n): "
    });
    if (answer.a !== "y") {
        return;
    }
    await job.moveToCompleted(returnValue, true);
    console.log(chalk_1.default.green(`Job "${jobId}" completed`));
});
exports.vorpal
    .command("clean <period>", `Clean queue for period ago, period format - ${msLink}`)
    .option("-s, --status <status>", "Status of the job to clean, default: completed")
    .option("-l, --limit <limit>", "Maximum amount of jobs to clean per call, default: all")
    .action(async function ({ period, options }) {
    await checkQueue();
    const answer = await this.prompt({
        name: "a",
        message: "Clean? (y/n): "
    });
    if (answer.a !== "y") {
        return;
    }
    const grace = period && period.length ? ms_1.default(period) : void 0;
    if (!grace) {
        return console.log(chalk_1.default.yellow("Incorrect period"));
    }
    const status = options.status || "completed";
    if (!["completed", "wait", "active", "delayed", "failed"].includes(status)) {
        return console.log(chalk_1.default.yellow("Incorrect status, should be: completed or wait or active or delayed or failed"));
    }
    const limit = Number.isInteger(options.limit) ? options.limit : void 0;
    await queue.clean(grace, status, limit);
    console.log(chalk_1.default.green(`Jobs cleaned`));
});
exports.vorpal
    .command("logs <jobId>", "get logs of job")
    .option("-s, --start <start>", "Start of logs")
    .option("-e, --end <end>", "End of logs")
    .action(async ({ jobId, options }) => {
    await checkQueue();
    const { logs, count } = await queue.getJobLogs(jobId, options.start, options.end);
    console.log(`Count of job logs: ${count}`);
    if (logs.length) {
        console.log("Logs:");
        logArray(logs);
    }
});
exports.vorpal
    .command("log <jobId> <data>", "add log to job")
    .action(async function ({ jobId, data }) {
    await checkQueue();
    const job = await getJob(jobId);
    const answer = await this.prompt({
        name: "a",
        message: "Add log? (y/n): "
    });
    if (answer.a !== "y") {
        return;
    }
    await job.log(data);
    console.log(chalk_1.default.green("Log added to job"));
});
exports.vorpal.history("bull-repl-default");
exports.vorpal.delimiter("BULL-REPL> ").show();
